// Teil des Projekts typisierte lineare abstrakte Datentypen -- nur ab Java (1.)5
// R. Krell (www.r-krell.de) -- 21.12.2006 -- Java JDK 1.5+ ntig

public abstract class TADT_StatSpeicherMitEndzugriff<Elementtyp> implements TADT_Speicher<Elementtyp>
// Gemeinsame Oberklasse fr Keller und Schlange

// Achtung: raus aus einem leeren Speicher liefert keinen (ArrayIndexOutOfBounds-) Fehler,
// sondern -- da Anfang und Ende der Reihung praktisch verbunden sind -- das Element
// an der nchsten (verbundenen) Stelle. Gibt's dort kein Element, wird null geliefert.
// Da dadurch kein Fehler bemerkt wird, wird kopf wie bei einer erfolgreichen raus-Operation
// verndert, sodass danach der Speicher nicht mehr als leer gilt und istLeer false
// liefert. Erst nach Einfgen eines Elements in den Speicher in diesem Zustands gilt dieser
// wieder als leer! Deshalb muss der Aufruf von raus bei leerem Speicher unbedingt vom
// Anwender vermieden werden!
{
  protected final int MAX = 100; // stat. Gre;  MAX-1 = Maximale Zahl der Elemente
  protected Elementtyp[] reihung ;  // Definition mit Elementtyp problemlos
  protected int kopf;
  
  public TADT_StatSpeicherMitEndzugriff()   // Konstruktor fr neuen, leeren Speicher
  {
    reihung = (Elementtyp[]) new Object [MAX]; // Anlegen der statischen Datenstruktur
      // Es ist allerdings nicht mglich, new Elementtyp[MAX] zu erzeugen. Es muss der
      // Umweg ber new Object[MAX] und den TypeCast gewhlt werden, der beim Compilieren
      // zur Warnung "unchecked cast" fhrt, weil der Cast mglicherweise erst zur
      // Laufzeit misslingt. Hier klappt's natrlich und die Warnung muss ignoriert werden.
    kopf = MAX-1;               // wegen raus;
  }
  
  public abstract void rein (Elementtyp element); // bergebenes Element kommt in Speicher
   // Methode wird von Keller und Schlange verschieden implementiert
   
  public abstract boolean istLeer();          // sagt, ob der Speicher leer ist
   // Methode wird von Keller und Schlange verschieden implementiert
   
  public Elementtyp zeige()     // zeigt oberstes Element ohne ndern des Speichers
  {
    return (reihung[kopf]);
  }
  
  public Elementtyp raus()      // zeigt oberstes El. und lscht es aus dem Speicher
  {
    Elementtyp hilf = reihung[kopf];
    kopf = (kopf+MAX-1) % MAX;  // reduziert kopf um 1. Nur nach 0 kommt wieder MAX-1
    return (hilf);
  }
}

